# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import smart_selects.db_fields


class Migration(migrations.Migration):

    dependencies = [
        ('test_app', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Book1',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=255)),
                ('publication', models.ForeignKey(to='test_app.Publication')),
                ('writer', smart_selects.db_fields.ChainedManyToManyField(chained_model_field=b'publications', to='test_app.Writer', chained_field=b'publication')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Location1',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('city', models.CharField(max_length=50)),
                ('street', models.CharField(max_length=100)),
                ('continent', models.ForeignKey(to='test_app.Continent')),
                ('country', smart_selects.db_fields.ChainedForeignKey(chained_model_field=b'continent', to='test_app.Country', chained_field=b'continent', auto_choose=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
    ]



# -*- coding: utf-8 -*-
from django.contrib import admin
from .models import Continent, Country, Location, Publication, Book, Writer, Grade, Team, Student


class ContinentAdmin(admin.ModelAdmin):
    list_display = ('name',)


class CountryAdmin(admin.ModelAdmin):
    list_display = ('name', 'continent')


class LocationAdmin(admin.ModelAdmin):
    list_display = ('continent', 'country', 'city', 'street')


class PublicationAdmin(admin.ModelAdmin):
    list_display = ('name',)


class BookAdmin(admin.ModelAdmin):
    list_display = ('name',)


class WriterAdmin(admin.ModelAdmin):
    list_display = ('name',)


class GradeAdmin(admin.ModelAdmin):
    list_display = ('name',)


class TeamAdmin(admin.ModelAdmin):
    list_display = ('name',)


class StudentAdmin(admin.ModelAdmin):
    list_display = ('name',)


admin.site.register(Continent, ContinentAdmin)
admin.site.register(Country, CountryAdmin)
admin.site.register(Location, LocationAdmin)
admin.site.register(Publication, PublicationAdmin)
admin.site.register(Book, BookAdmin)
admin.site.register(Writer, WriterAdmin)
admin.site.register(Grade, GradeAdmin)
admin.site.register(Team, TeamAdmin)
admin.site.register(Student, StudentAdmin)



import unittest

import django

from .db_fields import ChainedForeignKey, GroupedForeignKey


def has_new_migrations():
    return (django.VERSION[:2] >= (1, 7),
            "This test requires Django migrations introduced in Django 1.7.")


class AssertReconstructibleMixin(object):
    def assert_reconstructible(self, *field_args, **field_kwargs):
        field_instance = self.field_class(*field_args, **field_kwargs)
        name, path, args, kwargs = field_instance.deconstruct()
        new_instance = self.field_class(*args, **kwargs)

        for attr_name in self.deconstruct_attrs:
            self.assertEqual(
                getattr(field_instance, attr_name),
                getattr(new_instance, attr_name)
            )


@unittest.skipUnless(*has_new_migrations())
class ChainedForeignKeyTests(AssertReconstructibleMixin, unittest.TestCase):
    def setUp(self):
        self.field_class = ChainedForeignKey
        self.deconstruct_attrs = [
            'chained_field', 'chained_model_field', 'show_all', 'auto_choose',
            'view_name',
        ]

    def test_deconstruct_basic(self):
        self.assert_reconstructible(
            'myapp.MyModel',
            chained_field='a_chained_field',
            chained_model_field='the_chained_model_field',
            show_all=False, auto_choose=True
        )

    def test_deconstruct_mostly_default(self):
        self.assert_reconstructible(
            'myapp.MyModel'
        )

    def test_deconstruct_non_default(self):
        self.assert_reconstructible(
            'myapp.MyModel',
            chained_field='a_chained_field',
            chained_model_field='the_chained_model_field',
            show_all=True, auto_choose=True
        )


@unittest.skipUnless(*has_new_migrations())
class GroupedForeignKeyTests(AssertReconstructibleMixin, unittest.TestCase):
    def setUp(self):
        self.field_class = GroupedForeignKey
        self.deconstruct_attrs = ['group_field']

    def test_deconstruct_basic(self):
        self.assert_reconstructible('myapp.MyModel', 'the_group_field')



from django.core.urlresolvers import reverse
from django.test import TestCase, RequestFactory
from .models import Book, Country, Location, Student
from smart_selects.views import filterchain, filterchain_all


class ModelTests(TestCase):
    fixtures = ['chained_select', 'chained_m2m_select', 'grouped_select', 'user' ]

    def test_reverse_relationship_manager(self):
        cr = Country.objects.get(name='Czech republic')
        self.assertEqual(list(cr.location_set.all().values_list('city', flat=True)), ['Praha'])


class ViewTests(TestCase):
    fixtures = ['chained_select', 'chained_m2m_select', 'grouped_select', 'user' ]

    def setUp(self):
        self.factory = RequestFactory()
        self.assertTrue(self.client.login(username='admin', password='admin'))

    ############################ chained foreignkey ############################
    def test_location_add_get(self):
        response = self.client.get(reverse('admin:test_app_location_add'), follow=True)
        self.assertContains(response, 'Europe')
        self.assertContains(response, 'America')
        self.assertContains(response, 'var value = undefined;')

    def test_location_add_post(self):
        post_data = {
            'continent': '1',
            'country': '2',
            'city': 'New York',
            'street': 'Wallstreet',
        }
        self.client.post(reverse('admin:test_app_location_add'), post_data, follow=True)
        location = Location.objects.get(country__pk=2, continent__pk=1)
        self.assertEquals(location.city, 'New York')
        self.assertEquals(location.street, 'Wallstreet')

    def test_location_add_post_no_data(self):
        post_data = {
            'continent': '1',
            'country': '',
            'city': 'New York',
            'street': 'Wallstreet',
        }
        response = self.client.post(reverse('admin:test_app_location_add'), post_data)
        self.assertContains(response, 'This field is required.')
        self.assertContains(response, 'var value = undefined;')

    def test_location_change_get(self):
        response = self.client.get(reverse('admin:test_app_location_change', args=(1,)))
        self.assertContains(response, 'Europe')
        self.assertContains(response, 'var value = 1')

    def test_filterchain_view_for_chained_foreignkey(self):
        request = self.factory.get('')
        response = filterchain(request, 'test_app', 'Country', 'continent', 'test_app', 'Location', 'country', 1)
        expected_value = '[{"value": 1, "display": "Czech republic"}, {"value": 3, "display": "Germany"}, {"value": 4, "display": "Great Britain"}]'
        self.assertEquals(response.status_code, 200)
        self.assertJSONEqual(response.content.decode(), expected_value)

    def test_filterchain_all_view_for_chained_foreignkey(self):
        request = self.factory.get('')
        response = filterchain_all(request, 'test_app', 'Country', 'continent', 'test_app', 'Location', 'country', 1)
        expected_value = '[{"display": "Czech republic", "value": 1}, {"display": "Germany", "value": 3},'\
            ' {"display": "Great Britain", "value": 4}, {"display": "---------", "value": ""}, {"display": "New York", "value": 2}]'
        self.assertEquals(response.status_code, 200)
        self.assertJSONEqual(response.content.decode(), expected_value)

    def test_limit_to_choice_for_chained_foreignkey(self):
        request = self.factory.get('')
        # filterchain
        response = filterchain(request, 'test_app', 'Country', 'continent', 'test_app', 'Location1', 'country', 1)
        expected_value = '[{"value": 3, "display": "Germany"}, {"value": 4, "display": "Great Britain"}]'
        self.assertEquals(response.status_code, 200)
        self.assertJSONEqual(response.content.decode(), expected_value)
        # filterchain_all
        response = filterchain_all(request, 'test_app', 'Country', 'continent', 'test_app', 'Location1', 'country', 1)
        expected_value = '[{"value": 3, "display": "Germany"}, {"value": 4, "display": "Great Britain"}, {"display": "---------", "value": ""}]'
        self.assertEquals(response.status_code, 200)
        self.assertJSONEqual(response.content.decode(), expected_value)

    ############################ chained manytomany ############################
    def test_book_add_get(self):
        response = self.client.get(reverse('admin:test_app_book_add'))
        self.assertContains(response, 'Publication 1')
        self.assertContains(response, 'var value = null')

    def test_book_add_post(self):
        post_data = {
            'publication': '1',
            'writer': '2',
            'name': 'Book 2',
        }
        self.client.post(reverse('admin:test_app_book_add'), post_data, follow=True)
        book = Book.objects.get(writer__pk=2, publication__pk=1)
        self.assertEquals(book.name, 'Book 2')

    def test_book_add_post_no_data(self):
        post_data = {
            'publication': '1',
            'name': 'Book 2'
        }
        response = self.client.post(reverse('admin:test_app_book_add'), post_data)
        self.assertContains(response, 'This field is required.')
        self.assertContains(response, 'var value = []')

    def test_book_change_get(self):
        response = self.client.get(reverse('admin:test_app_book_change', args=(1,)), follow=True)
        self.assertContains(response, 'var value = [3];')

    def test_filterchain_view_for_chained_manytomany(self):
        request = self.factory.get('')
        response = filterchain(request, 'test_app', 'Writer', 'publications', 'test_app', 'Book', 'writer', 1)
        expected_value = '[{"display": "Author 3", "value": 3}]'
        self.assertEquals(response.status_code, 200)
        self.assertJSONEqual(response.content.decode(), expected_value)

    def test_limit_to_choice_for_chained_manytomany(self):
        request = self.factory.get('')
        # filterchain
        response = filterchain(request, 'test_app', 'Writer', 'publications', 'test_app', 'Book1', 'writer', 1)
        expected_value = '[]'
        self.assertEquals(response.status_code, 200)
        self.assertJSONEqual(response.content.decode(), expected_value)

    ############################ grouped foreignkey ############################
    def test_student_add_get(self):
        response = self.client.get(reverse('admin:test_app_student_add'))
        self.assertContains(response, '<optgroup label="Grade 1">\n<option value="1">   Team 1</option>\n</optgroup>')
        self.assertContains(response, '<optgroup label="Grade 2">\n<option value="2">   Team 2</option>\n</optgroup>')

    def test_student_add_post(self):
        post_data = {
            'name': 'Student 2',
            'grade': 2,
            'team': 2
        }
        response = self.client.post(reverse('admin:test_app_student_add'), post_data)
        student = Student.objects.get(grade=2, team=2)
        self.assertEquals(student.name, 'Student 2')
    


# -*- coding: utf-8 -*-

from django.utils.encoding import force_text
try:
    from django.apps import apps
    get_model = apps.get_model
except ImportError:
    from django.db.models.loading import get_model


def unicode_sorter(input):
    """ This function implements sort keys for the german language according to
    DIN 5007."""

    # key1: compare words lowercase and replace umlauts according to DIN 5007
    key1 = input.lower()
    key1 = key1.replace(u"ä", u"a")
    key1 = key1.replace(u"ö", u"o")
    key1 = key1.replace(u"ü", u"u")
    key1 = key1.replace(u"ß", u"ss")

    # key2: sort the lowercase word before the uppercase word and sort
    # the word with umlaut after the word without umlaut
    # key2=input.swapcase()

    # in case two words are the same according to key1, sort the words
    # according to key2.
    return key1


def get_limit_choices_to(app_name, model_name, field_name):
    try:
        model = get_model(app_name, model_name)
        field = model._meta.get_field(field_name)
        limit_choices_to = field.rel.limit_choices_to
    except Exception:
        limit_choices_to = None

    return limit_choices_to


def get_queryset(model_class, manager=None, limit_choices_to=None):
    if manager is not None and hasattr(model_class, manager):
        queryset = getattr(model_class, manager)
    else:
        queryset = model_class._default_manager

    if limit_choices_to:
        queryset = queryset.complex_filter(limit_choices_to)
    return queryset


def serialize_results(results):
    return [
        {'value': item.pk, 'display': force_text(item)} for item in results
    ]


def get_keywords(field, value, m2m=False):
    if value == '0':
        keywords = {str("%s__isnull" % field): True}
    elif m2m:
        keywords = {str("%s__pk" % field): str(value)}
    else:
        keywords = {str(field): str(value)}

    return keywords


def sort_results(results):
    """Performs in-place sort of filterchain results."""

    results.sort(key=lambda x: unicode_sorter(force_text(x)))



from setuptools import setup, find_packages


setup(name="django-smart-selects",
      version="1.2.9",
      description="Django application to handle chained model fields.",
      author="Patrick Lauber",
      author_email="digi@treepy.com",
      url="https://github.com/digi604/django-smart-selects",
      packages=find_packages(),
      include_package_data=True,
      )
